/**
 * 
 */
package de.uni_stuttgart.fmi.szs.jmoped.coverage;

import org.gjt.jclasslib.bytecode.AbstractInstruction;

public class AnnotatedInstruction implements Comparable
{
	private AbstractInstruction instruction;
	private int postStarCount = 0;
	private int preStarCount = 0;
	private CoverageMethod method;
	
	public AnnotatedInstruction(AbstractInstruction instruction, CoverageMethod method)
	{
		this.instruction = instruction;
		this.method = method;
	}
	
	public int getOffset()
	{
		return instruction.getOffset();
	}
	
	public AbstractInstruction getInstruction()
	{
		return instruction;
	}
	
	void markPostStarCovered()
	{
		postStarCount++;
	}
	
	public int getPostStarCovered()
	{
		return postStarCount;
	}
	
	void markPreStarCovered()
	{
		preStarCount++;
	}
	
	public int getPreStarCovered()
	{
		return preStarCount;
	}
	
	public int getCovered()
	{
		return Math.max(postStarCount, preStarCount);
	}
	
	public boolean isCovered()
	{
		return getCovered() > 0;
	}
	
	public String getLabel()
	{
		int offset = getOffset(); 
		if (offset == 0) {
			return method.getMethod().getFormattedName();
		}
		else {
			return method.getMethod().getFormattedName() + offset;
		}
	}

	public int compareTo(Object o) 
	{
		if (o instanceof Integer) {
			return  instruction.getOffset() - (Integer)o;
		}
		return 0;
	}
	
}