package de.uni_stuttgart.fmi.szs.jmoped.coverage;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import org.gjt.jclasslib.structures.InvalidByteCodeException;

import de.uni_stuttgart.fmi.szs.jmoped.PDS;
import de.uni_stuttgart.fmi.szs.jmoped.PDSClass;
import de.uni_stuttgart.fmi.szs.jmoped.PDSMethod;

public class CoverageUtils {

	public static CoverageMethod getMethod(PDS pds, String label) 
	{
		label = getMethodLabel(label);
		PDSClass clazz = null;
		for (PDSClass c : pds.getPDSClassList()) {
			if (label.startsWith(c.getFormattedClassName())) {
				clazz = c;
				break;
			}
		}
		if (clazz == null) {
			return null;
		}
		for (PDSMethod m : clazz.getMethodList()) {
			if (label.startsWith(m.getFormattedName())) {
				return new CoverageMethod(m);
			}
		}
		return null;
	}

	public static String getMethodLabel(String label) {
		int offset = getOffsetPosition(label);
		if (offset >= 0) {
			return label.substring(0, offset);
		} else {
			return label;
		}
	}

	/**
	 * Returns the position of the first digit of the offset number.
	 * @return -1 if there is none
	 */
	public static int getOffsetPosition(String label) {
		int offset = label.length() - 1;
		while (offset >= 0 && Character.isDigit(label.charAt(offset))) {
			offset--;
		}
		// at least one digit
		if (offset < label.length() - 1) {
			return offset + 1;
		}
		return -1;
	}
	
	/**
	 * Extracts the offset number at the end of the label string.
	 * @param label
	 * @return o if there is no number
	 */
	public static int extractOffset(String label)
	{
		int offset = getOffsetPosition(label);
		if (offset >= 0) {
			return Integer.parseInt(label.substring(offset));
		}
		return 0;
	}

	/**
	 * Writes the Remopla code of <code>pds</code> to a unique temporary file having 
	 * the prefix name <code>fileName</code>.
	 *
	 * @return the temporary file on success
	 * @throws InvalidByteCodeException if the Remopla code could not be generated
	 * @throws IOException if there was an error writing to the temporary file
	 */
	public static File writeRemopla(PDS pds, String fileName)
			throws InvalidByteCodeException, IOException {
		String mopedInput = pds.toRemopla();
		BufferedWriter out = null;
		try {
			File tmpFile = File.createTempFile(fileName, ".rem");
//			tmpFile.deleteOnExit();
			out = new BufferedWriter(new FileWriter(tmpFile));
			out.write(mopedInput);
			return tmpFile;
		} finally {
			if (out != null) {
				try {
					out.close();
				} catch (IOException ie) {
				}
			}
		}
	}
}
