package de.uni_stuttgart.fmi.szs.jmoped.coverage;

import java.util.ArrayList;

public class DescriptorParser
{

	private static final int INITIAL = 0;
	
	private static final int START = 1;
	
	private static final int IN_PARAM = 2;
	
	private static final int DONE = 3;
	
	public static Parameter[] parse(String descriptor)
	{
		ArrayList<Parameter> params = new ArrayList<Parameter>(3);
		int state = INITIAL;
		char[] input = descriptor.toCharArray();
		int index = 0;
		int typeStart = 0;
		
		ParameterImpl current = new ParameterImpl();

		while (state != DONE) {
			switch (state) {
			case INITIAL:
				if (input[index] != '(') {
					throw new IllegalStateException();
				}
				index++;
				state = START;
				break;
			case START:
				switch (input[index]) {
				case ')':
					state = DONE;
					break;
				case '[':
					current.isArray = true;
					break;
				case 'I':
				case 'Z':
					current.isValue = true;
					current.type = String.valueOf(input[index]);
					params.add(current);
					current = new ParameterImpl();
					break;
				case 'L':
					state = IN_PARAM;
					typeStart = index + 1;
					break;
				default:
					throw new IllegalStateException("unhandled parameter" + input[index]);
				}
				index++;
				break;
			case IN_PARAM:
				if (input[index] == ';') {
					current.type = new String(input, typeStart, index - typeStart);
					params.add(current);
					current = new ParameterImpl();
					state = START;
				}
				index++;
			}
		}
		return params.toArray(new Parameter[params.size()]);
	}
	
	private static class ParameterImpl implements Parameter
	{
		boolean isValue;
		boolean isArray;
		boolean isReferenceArray;
		String type;
		
		public boolean isValue()
		{
			return isValue && !isArray;
		}

		public boolean isValueArray() 
		{
			return isValue && isArray;
		}

		public boolean isReference()
		{
			return !isValue && !isArray;
		}

		public boolean isReferenceArray() 
		{
			return !isValue && isArray;
		}
		
		public String getType()
		{
			return type;
		}
		
		@Override
		public String toString() 
		{
			if (isValue()) {
				if (type.equals("Z")) {
					return "boolean";
				}
				else {
					return "int";
				}
			}
			else if (isValueArray()) {
				if (type.equals("Z")) {
					return "boolean array";
				}
				else {
					return "int array";
				}
			}
			else if (isReference()) {
				return "reference to " + type;
			}
			else {
				return "reference to " + type + " array";
			}
		}
	}
	
}
