package de.uni_stuttgart.fmi.szs.jmoped.coverage;

public class IntConfig 
{
	
	static {
		System.loadLibrary("mopedcoverage");
	}
	
	private final int handle;
	
	private String stackSymbol = null;
	
	/**
	 * IntConfigs created from a previous have point to it, so it is not deleted on the
	 * native as long as references to the children exist.
	 */
	private final IntConfig previous;

	private IntConfig next = null;
	
	private boolean hasNext = true;
	
	private int stackLength = -1;
	
	private IntVariable[] stackVars = null;
	
	private IntVariable[] globalVars = null;
	
	public IntConfig(int handle)
	{
		this(handle, null);
	}
	
	public IntConfig(int handle, IntConfig parent)
	{
		this.handle = handle;
		this.previous = parent;
	}
	
	public IntVariable[] getGlobalVariables()
	{
		if (globalVars == null) {
			synchronized (Coverage.class) {
				globalVars = getGlobalVariablesNative();
				if (globalVars == null) {
					globalVars = new IntVariable[0];
				}
			}
		}
		return globalVars;
	}
	
	private native IntVariable[] getGlobalVariablesNative();
	
	public IntVariable[] getStackVariables()
	{
		if (stackVars == null) {
			synchronized (Coverage.class) {
				stackVars = getStackVariablesNative();
				if (stackVars == null) {
					stackVars = new IntVariable[0];
				}
			}
		}
		return stackVars;
	}
	
	private native IntVariable[] getStackVariablesNative();
	
	public IntConfig getConfigAt(String label)
	{
		if (getStackSybmol().equals(label)) {
			return this;
		}
		
		IntConfig nextConfig = getNextConfig();
		if (nextConfig != null) {
			return nextConfig.getConfigAt(label);
		}
		
		return null;
	}
	
	private native IntConfig getConfigAtNative(String label);

	public String getStackSybmol()
	{
		if (stackSymbol == null) {
			synchronized (Coverage.class) {
				stackSymbol = getStackSymbolNative();
			}
		}
		return stackSymbol;
	}
	
	private native String getStackSymbolNative();
	
	/**
	 * @return <code>null</code> if there is no next config.
	 */
	public IntConfig getNextConfig()
	{
		if (next == null && hasNext) {
			synchronized (Coverage.class) {
				next = getNextConfigNative();
				hasNext = false;
			}
		}
		return next;
	}
	
	/**
	 * 
	 * @return null if this the start configuration.
	 */
	public IntConfig getPreviousConfig()
	{
		return previous;
	}
	
	private native IntConfig getNextConfigNative();
	
	public int getStackLength()
	{
		if (stackLength == -1) {
			synchronized (Coverage.class) {
				stackLength = getStackLengthNative();
			}
		}
		return stackLength;
	}
	
	private native int getStackLengthNative();
}
