package de.uni_stuttgart.fmi.szs.jmoped.coverage;

import java.util.ArrayList;

import de.uni_stuttgart.fmi.szs.jmoped.PDS;
import de.uni_stuttgart.fmi.szs.jmoped.coverage.CoverageMethod.State;

public class MulticastCoverageListener implements CoverageListener
{

	private ArrayList coverageListeners = new ArrayList();
	
	/**
	 * Adds a coverage listener that is notified of the test progress.
	 * @param listener
	 */
	public void addCoverageListener(CoverageListener listener)
	{
		synchronized (coverageListeners) {
			coverageListeners.add(listener);
		}
	}
	
	/**
	 * Removes the coverage listener.
	 * @param listener
	 */
	public void removeCoverageListener(CoverageListener listener)
	{
		synchronized (coverageListeners) {
			coverageListeners.remove(listener);
		}
	}
	
	public void coverageStarted(PDS pds)
	{
		CoverageListener[] listeners = getListeners();
		for (int i = listeners.length - 1; i >= 0; i--) {
			listeners[i].coverageStarted(pds);
		}
	}
	
	public void coverageEnded(CoverageResult result)
	{
		CoverageListener[] listeners = getListeners();
		for (int i = listeners.length - 1; i >= 0; i--) {
			listeners[i].coverageEnded(result);
		}
	}
	
	public void coverageFailed(final String message, final Throwable t)
	{
		CoverageListener[] listeners = getListeners();
		for (int i = listeners.length - 1; i >= 0; i--) {
			listeners[i].coverageFailed(message, t);
		}
	}
	
	public void assertErrorReachedFrom(CoverageMethod method, int offset)
	{
		CoverageListener[] listeners = getListeners();
		for (int i = listeners.length - 1; i >= 0; i--) {
			listeners[i].assertErrorReachedFrom(method, offset);
		}
	}
	
	public void assertErrorReachedFrom(CoverageMethod method, int offset, 
			Transition transition)
	{
		CoverageListener[] listeners = getListeners();
		for (int i = listeners.length - 1; i >= 0; i--) {
			listeners[i].assertErrorReachedFrom(method, offset, transition);
		}
	}
	
	public void coveredMethodOffset(CoverageMethod method, int offset,
			State state)
	{
		CoverageListener[] listeners = getListeners();
		for (int i = listeners.length - 1; i >= 0; i--) {
			listeners[i].coveredMethodOffset(method, offset, state);
		}
	}
	
	public CoverageListener[] getListeners()
	{
		synchronized (coverageListeners) {
			return (CoverageListener[])coverageListeners.toArray(new CoverageListener[0]);
		}
	}

	public void heapOverflow(CoverageMethod method, int lineOffset)
	{
		CoverageListener[] listeners = getListeners();
		for (int i = listeners.length - 1; i >= 0; i--) {
			listeners[i].heapOverflow(method, lineOffset);
		}
	}

	public void labelReached(String label) 
	{
		CoverageListener[] listeners = getListeners();
		for (int i = listeners.length - 1; i >= 0; i--) {
			listeners[i].labelReached(label);
		}
	}
	
}
