package de.uni_stuttgart.fmi.szs.jmoped.coverage;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.gjt.jclasslib.structures.InvalidByteCodeException;

import de.uni_stuttgart.fmi.szs.jmoped.PDS;
import de.uni_stuttgart.fmi.szs.jmoped.PDSMethod;

public class PostStarReachabilityTester extends Coverage 
{
	
	public PostStarReachabilityTester(Class clazz) throws Exception
	{
		this(clazz, new String[0]);
	}
	
	/**
	 * Convenience construtor for {@link #CoverageTester(String, String[], String)
	 * PostStarReachabilityTester(clazz.getName(), searchPaths, methodName)}.
	 */
	public PostStarReachabilityTester(Class clazz, String[] searchPaths)
			throws Exception {
		this(clazz.getName(), searchPaths);
	}

	/**
	 * Constructs a PostStarReachabilityTester for class <code>className</code>.
	 * <p>
	 * 
	 * @param className should be the fully qualified name of the class as it
	 * is returned by {@link Class#getName()}.
	 * @param searchPaths array of names of directories and jar files that are
	 * searched for the class file of <code>className</code>
	 * @param methodName the method that should be tested for coverage. The format
	 * should match {@link PDSMethod#getFormattedName()}, can be <code>null</code>
	 * @throws Exception
	 */
	public PostStarReachabilityTester(String className, String[] searchPaths)
			throws Exception 
	{
		this(new PDS(className, searchPaths));
	}

	public PostStarReachabilityTester(PDS pds) 
	{
		super(pds);
	}
	
	/**
	 * Test all instructions of <code>method</code> for reachability.
	 * @param method
	 */
	public void test(CoverageMethod method)
	{
		List<AnnotatedInstruction> instructions = method.getAnnotatedInstructions();
		ArrayList<String> labels = new ArrayList<String>(instructions.size());
		for (AnnotatedInstruction i : instructions) {
			labels.add(i.getLabel());
		}
		test(labels.toArray(new String[0]));
	}
	
	public void test(String label) 
	{
		test(new String[] { label });
	}

	public void test(String label, ProgressMonitor monitor)
	{
		test(new String[] { label }, monitor);
	}
	
	public void test(String[] labels) 
	{
		test(labels, new NullProgressMonitor());
	}

	public void test(String[] labels, ProgressMonitor monitor) 
	{
		clearMethodsMap();
		CoverageListener listenerSupport = getListenerSupport();

		listenerSupport.coverageStarted(getPDS());
		monitor.beginTask("Post Star Analysis", 10000);
		
		File tmpFile = null;
		try {
			tmpFile = CoverageUtils.writeRemopla(getPDS(), labels[0]);
		} catch (InvalidByteCodeException ibce) {
			listenerSupport.coverageFailed("Invalid bytecode", ibce);
			monitor.done();
			return;
		} catch (IOException ie) {
			listenerSupport.coverageFailed("Error writing to temporary file "
					+ tmpFile, ie);
			monitor.done();
			return;
		}
		CoverageResult result;
		synchronized (super.getClass()) {
			if (monitor.isCanceled()) {
				monitor.done();
				return;
			}
			result = analyseReachability(tmpFile.getAbsolutePath(), labels, 
					new SubProgressMonitor(10000, monitor));
		}
		if (monitor.isCanceled()) {
			listenerSupport.coverageFailed("Post Star Coverage was cancelled",
					new InterruptedException());
		} 
		else {
			result.setPDS(getPDS());
			listenerSupport.coverageEnded(result);
		}
		monitor.done();
	}

	private native CoverageResult analyseReachability(String fileName, String[] labels,
			ProgressMonitor monitor);

}
