package de.uni_stuttgart.fmi.szs.jmoped.coverage;

import de.uni_stuttgart.fmi.szs.jmoped.PDS;
import de.uni_stuttgart.fmi.szs.jmoped.PDSInfo;
import de.uni_stuttgart.fmi.szs.jmoped.PDSMethod;

class ReferenceMethodArgument implements MethodArgument
{
	private Parameter param;
	
	private IntVariable var;
	
	private IntConfig config; 
	
	private PDS pds;
	
	MethodCall constructorCall;
	
	public ReferenceMethodArgument(Parameter param, IntVariable var, 
			IntConfig config, PDS pds)
	{
		this.param = param;
		this.var = var;
		this.config = config;
		this.pds = pds;
	}

	public boolean isValue() 
	{
		return false;
	}

	/**
	 * Returns the variable that points to the heap where the object is.
	 */
	public IntVariable getVariable() 
	{
		return null;
	}

	public boolean isArray() 
	{
		return false;
	}

	public int[] getArray() 
	{
		return null;
	}

	public boolean isReference() 
	{
		return true;
	}

	public Parameter getFormalParameter() 
	{
		return param;
	}
	
	public MethodCall getConstructorCall() 
	{
		if (constructorCall == null) {
			constructorCall = resolveConstructorCall();
		}
		return constructorCall;
	}

	private MethodCall resolveConstructorCall()
	{
		String className = param.getType();
		String constructorPrefix = 
			PDSMethod.formatMethodName(new String[] { className, "<init>", "" });
		int heapIndex = var.intValue();
		String heapPtrName = pds.getInfo().getHeapPtrName();
		IntConfig previous = config.getPreviousConfig();
		
		// go backwards through configs and find matching constructor all
		while (previous != null) {
			if (previous.getStackSybmol().startsWith(constructorPrefix)) {
				IntVariable[] globalVars = previous.getGlobalVariables();
				IntVariable heapPtr = 
					JavaMethodArgumentsResolver.findVariable(globalVars, heapPtrName);
				if (heapPtr != null && heapPtr.intValue() == heapIndex) {
					break;
				}
			}
			previous = previous.getPreviousConfig();
		}
		
		if (previous != null) {
			JavaMethodArgumentsResolver resolver = new JavaMethodArgumentsResolver();
			PDSMethod method = pds.getMethodByFormattedName(previous.getStackSybmol());
			resolver.resolveMethodCalls(previous, method, pds);
		}
		return null;
	}
	
	
	
}