package de.uni_stuttgart.fmi.szs.jmoped.coverage;

public class SubProgressMonitor implements ProgressMonitor 
{

	private ProgressMonitor parent;
	private int totalTicks;
	private int ticks;
	private int totalAmount;
	private int amount;
	
	public SubProgressMonitor(int totalTicks, ProgressMonitor parent)
	{
		this.totalTicks = totalTicks;
		this.parent = parent;
		if (parent == null) {
			throw new NullPointerException("parent must not be null");
		}
	}
	
	public void beginTask(String description, int workAmount) 
	{
		if (workAmount <= 0) {
			throw new IllegalArgumentException("workAmount must be > 0");
		}
		totalAmount = workAmount;
		parent.subTask(description);
	}

	public void done() 
	{
		if (!isCanceled() && ticks < totalTicks) {
			parent.worked(totalTicks - ticks);
			ticks = totalTicks;
		}
	}

	public boolean isCanceled() 
	{
		return parent.isCanceled();
	}

	public void setCanceled(boolean cancel) 
	{
		throw new UnsupportedOperationException("must not be called on a submonitor");
	}

	public void subTask(String description) 
	{
		parent.subTask(description);
	}

	public void worked(int work) 
	{
		amount += work;
		int newTicks = amount * totalTicks / totalAmount;
		if (newTicks > ticks) {
			parent.worked(newTicks - ticks);
			ticks = newTicks;
		}
	}

}
