package de.uni_stuttgart.fmi.szs.jmoped.ex;

import de.uni_stuttgart.fmi.szs.jmoped.annotation.*;

/**
 * This class contains two versions of Quicksort. Both were found by
 * using Google with "java quicksort".
 * 
 * @author suwimodh
 *
 */
public class QuickSort {
	
	/*
	 * The first version 
	 */
    static void qsort(int a[], int lo0, int hi0) {
        int lo = lo0;
        int hi = hi0;
        if (lo >= hi) {
            return;
        }
        int mid = a[(lo + hi) / 2];
        while (lo < hi) {
            while (lo<hi && a[lo] < mid) {
                lo++;
            }
            while (lo<hi && a[hi] >= mid) {
                hi--;
            }
            if (lo < hi) {
                int T = a[lo];
                a[lo] = a[hi];
                a[hi] = T;
            }
        }
        if (hi < lo) {
            int T = hi;
            hi = lo;
            lo = T;
        }
        qsort(a, lo0, lo);
        qsort(a, lo == lo0 ? lo+1 : lo, hi0);
    }

    /*
     * Driver of the first version
     */
    static void sort(int a[]) {
        qsort(a, 0, a.length-1);
    }
    
    /**
     * Checks whether the array is correctly sorted 
     * 
     * @param array the array.
     * @return true if the array is sorted, false otherwise.
     */
	static boolean isSorted(int[] array)
	{
		for (int i = 0; i < array.length - 1; i++) {
			if (array[i] > array[i+1]) {
				return false;
			}
		}
		return true;
	}
	
    /*
     * The second version
     */
	static void quicksort (int[] a, int lo, int hi)
	{
//	  lo is the lower index, hi is the upper index
//	  of the region of array a that is to be sorted
		int i=lo, j=hi, h;
	    int x=a[(lo+hi)/2];

	    //  partition
	    do
	    {    
	        while (a[i]<x) i++; 
	        while (a[j]>x) j--;
	        if (i<=j)
	        {
	            h=a[i]; a[i]=a[j]; a[j]=h;
	            i++; j--;
	        }
	    } while (i<=j);

	    //  recursion
	    if (lo<j) quicksort(a, lo, j);
	    if (i<hi) quicksort(a, i, hi);
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
	
		// Creates an array
		int[] a = new int[3];
		for (int i = 0; i < a.length; i++) {
			a[i] = Integer.parseInt(args[0]);
		}
		
		// Counterexample for the first version
		// int[] a = {1, 0, 1};
		
		// Checks the first version
		sort(a);
		assert(isSorted(a));
		
		// Checks the second version
		//quicksort(a, 0, a.length-1);
		//assert(isSorted(a));
	}
}
