/*
    Copyright (C) 2005 Dejvuth Suwimonteerabuth

    This file is part of jMoped.

    jMoped is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    jMoped is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with jMoped; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package de.uni_stuttgart.fmi.szs.jmoped;

import static java.io.File.separator;

import java.io.*;
import java.util.Hashtable;

class jmoped {
	
	static void usage(String name) {
		
		System.err.println(
			"\n" + name + " -- java model checker\n\n" +
			
			"Usage: " + name + " [-<options>] <classfile>\n\n" +
			
			"    Options:\n" +
			"    -a\t\t check violation of assertion\n" +
			"    -e\t\t check NullPointerException\n" +
			"    -h\t\t check heap overflow\n" +
			"    -n\t\t create rem file for checking NullPointerException\n" +
			"    -o<..>\t set output file name\n" +
			"    -r\t\t check reachability\n" +
			"    -s\t\t simplify output (only works with simple examples)\n" +
			"    -t\t\t use different 1-bit heap strategy\n\n" +
			
			"    Options that overwrite jmoped.conf:\n" + 
			"    --bits=<n>\t set number of bits of integer variables\n" +
			"    --heap=<n>\t set size of the heap\n" +
			"    --line<l>=<n> set number of bits of array at line <l>\n" +
			"    Note that 2^bits must be greater than the heap size.\n" +
			"    Otherwise, ceil(log_2(heap+1)) is used instead.\n\n" +
			
			"Copyright (C) 2004-2006 \n" + 
			"Dejvuth Suwimonteerabuth " +
			"<suwimodh@fmi.uni-stuttgart.de>\n\n" +
			"See COPYING for legal information.\n\n" +
			
			"Version 2.2.3 (17.11.2006)\n"
			);
		
		System.exit(1);
	}
	
	static void optError() {
		
		System.err.println("Options -a and -r cannot mix together\n");
		System.exit(1);
	}
	
	public static void main(String[] args) throws Exception {
		
		String toolName = "jmoped";
		String inFileName = "";
		String outFileName = "";
		String classFileName;
		String mopedForm = "";
		int bits = 0;
		int heapSize = 0;
		boolean chkAssert = false;
		boolean chkHeap = false;
		boolean chkNPE = false;
		boolean chkReach = false;
		boolean enableNPEChecks = false;
		boolean twoDims = false;
		Hashtable<Integer, Integer> lineBitTable = 
			new Hashtable<Integer, Integer>();
		
		if (args.length == 0) {
			usage(toolName);
		}
		
		for (int i = 0; i < args.length; i++) {
			if (args[i].startsWith("-")) {
				for (int j = 1; j < args[i].length(); j++) {
					switch (args[i].charAt(j)) {
						
						case 'a': 
							if (chkReach) {
								optError();
							}
							chkAssert = true;
							break;
						
						case 'e': 
							enableNPEChecks = true;
							chkNPE = true;
							break;
							
						case 'h': 
							chkHeap = true;
							break;
							
						case 'n': 
							enableNPEChecks = true;
							break;
						
						case 'o': 
							outFileName = args[i].substring(j+1);
							j = args[i].length();
							break;
						
						case 'r': 
							if (chkAssert) {
								optError();
							}
							chkReach = true;
							break;
							
						case 's':
							PDSInfo.simplified = true;
							break;
							
						case 'S':
							PDSInfo.Stefan = true;
							break;
						
						case 't':
							twoDims = true;
							break;
							
						case '-':
							if (args[i].substring(j+1, j+6).equals("bits=")) {
								try {
									bits = Integer.parseInt(args[i].substring(j+6));
								} catch (NumberFormatException nfe) {
									usage(toolName);
								}
								j = args[i].length();
							} else if (args[i].substring(j+1, j+6).equals("heap=")) {
								try {
									heapSize = Integer.parseInt(args[i].substring(j+6));
								} catch (NumberFormatException nfe) {
									usage(toolName);
								}
								j = args[i].length();
							} else if (args[i].substring(j+1, j+5).equals("line")) {
								try {
									int equalIndex = args[i].indexOf('=', j+5);
									int l = Integer.parseInt(args[i].substring(j+5, equalIndex));
									int b = Integer.parseInt(args[i].substring(equalIndex + 1));
									lineBitTable.put(l,b);
								} catch (NumberFormatException nfe) {
									usage(toolName);
								}
								j = args[i].length();
							} else 
								usage(toolName);
							break;

						default: 
							usage(toolName);
							break;
					}
				}
			} else {
				if (inFileName.length() == 0) {
					inFileName = args[i];
				} else if (chkReach && (mopedForm.length() == 0)) {
					mopedForm = args[i];
				} else {
					System.out.println("b");
					usage(toolName);
				}
			}
		}
		
		//Formats input file name
		if (inFileName.length() == 0) {
			usage(toolName);
		}
		
		//If input is Java source, compiles it.
		if (inFileName.endsWith(".java")) {
			
			//Adds tools.jar to classpath
			String toolsString = PDSDefault.JAVA_HOME + separator + "lib" 
					+ separator + "tools.jar";
			File toolsFile = new File(toolsString);
			if (!toolsFile.exists()) {
				System.out.println("Cannot find " + toolsString + "."); 
				System.out.println("Make sure that you have JDK installed.");
				System.out.println("Otherwise, contact suwimodh@fmi.uni-stuttgart.de.");
				System.exit(1);
			}
			PDSDefault.addClasspath(toolsFile);
			
			System.out.println("Compiling " + inFileName + " ..");
			classFileName = inFileName.substring(0,inFileName.length() - 5)
				+ ".class";
			String[] javacArgs = new String[] {"-g", inFileName};
			int status = com.sun.tools.javac.Main.compile(javacArgs);
			if (status != 0) {
				System.exit(1);
			}
		} else if (!inFileName.endsWith(".class")) {
			classFileName = inFileName.concat(".class");
		} else {
			classFileName = inFileName;
		}
		
		//Translation Info
		System.out.println("Translating ..");
		PDS pds = new PDS(classFileName);
		System.out.println("\nThe following methods are going to be translated:");
		System.out.println(pds.includedTableInfo());
		
		//Changes sizes of integer and heap
		if (bits > 0)
			pds.setBits(bits);
		if (heapSize > 0)
			pds.setHeapSize(heapSize);
		if (twoDims)
			pds.setHeapOption(PDSInfo.HeapOption.TWODIMS);
		if (!lineBitTable.isEmpty())
			pds.putAllLineBitTable(lineBitTable);
		
		pds.getInfo().setCheckForNullPointerExceptions(enableNPEChecks);
		
		//Translation
		pds.translate();
		
		//Writes the resulting PDS to a file
		if (outFileName.length() == 0) {
			outFileName = classFileName.replaceAll(".class", ".rem");
		}
		File outFile = new File(outFileName);
		Writer output = null;
		try {
			output = new BufferedWriter(new FileWriter(outFile));
			output.write(pds.toRemopla());
		}
		finally {
			if (output != null) output.close();
		}
		System.out.println("Translation completed.");
		
		//Runs moped
		if (chkAssert || chkHeap || chkNPE || chkReach)  {
			
			String[] mopedCmd = new String[3];
			mopedCmd[0] = pds.getMopedPath();
			mopedCmd[1] = outFileName;
			if (chkAssert) {
				mopedCmd[2] = pds.getLabelAssertError();
			} else if (chkHeap) {
				mopedCmd[2] = pds.getLabelHeapOverflow();
			} else if (chkNPE) {
				mopedCmd[2] = pds.getLabelNPE();
			} else if (chkReach) {
				mopedCmd[2] = mopedForm;
			}
			
			System.out.println("Running moped ..");
			System.out.println(mopedCmd[0] + " " + mopedCmd[1] + " " 
				+ mopedCmd[2]);
			Runtime runtime = Runtime.getRuntime();
			StringBuffer returnVal = new StringBuffer();
			String buffer;
			try {
				Process process = runtime.exec(mopedCmd);
				
				BufferedReader in = new BufferedReader(new InputStreamReader(
					process.getInputStream()));
				BufferedReader err = new BufferedReader(new InputStreamReader(
					process.getErrorStream()));
				
				while((buffer = in.readLine()) != null) {
					returnVal.append(buffer).append("\n");
				}
				while((buffer = err.readLine()) != null) {
					returnVal.append(buffer).append("\n");
				}
				
				System.out.println(returnVal);
			} catch (Exception e) {
				System.out.println(e);
			}
		}
	}
}
